% DebugTK
%   Utility to help debugging FOGP
%
% October 26, 2014
%   handle of FFTIFO itself is saved in all manager objects, 
%   instead of handle of multiple manager handles
%
% October 22, 2014
%   Make a standard object
%   Open(fileName) starts recording and close stops recording
% 
classdef DebugTK < handle
    
    properties
        ifo   % handle of this FFTIFO
        
        title % title of this debug, prepanded to all messages
        fID   % file ID, ~=0 if file opened
        maxRec;  % keep this many of records
        counter; % record counter
    end
    
    methods
       % constructors
        function obj = DebugTK( ifo )
            obj.ifo = ifo;
            
            obj.title = class( ifo );
            obj.fID = 0; % no file opened
            obj.maxRec = 500;
        end
        
        function ID = on( obj, fileName )
            % open output log file or to command window

            obj.off;
            if exist( 'fileName', 'var' )
                fileName = strtrim(fileName);
                if isempty( fileName )
                    obj.fID = 1;
                else
                    [obj.fID, msg] = fopen( fileName, 'w' );
                    if obj.fID == -1
                        error('File open error : ''%s''', msg);
                        obj.fID = 0;
                    end
                end
            else
                obj.fID = 1;
            end
            if obj.fID == 1
                msg = 'to command window';
            else
                msg = sprintf('to file ''%s''', fileName);
            end
            obj.reset;
            fprintf('Debug stream opened %s\n', msg);
            ID = obj.fID;
        end
        
        function reset( obj, maxRec )
            if exist( 'maxRec', 'var' )
                obj.maxRec = maxRec;
            end
            obj.counter = 0;
        end
        
        function off( obj )
            % close the outpout stream and set the fileID to 0
            ID = obj.fID;
            obj.fID = 0;  % in case fclose fails, 
            if ID ~= 0 && ID ~= 1
                fclose( ID );
            end
            fprintf('Debug stream closed\n');
        end
            
        function print( obj, varargin )
            % if obj.fID ~= 0, printout the argument to the output file or to command widnow
            obj.counter = obj.counter + 1;
            if ((obj.maxRec < 0) | (obj.counter <= obj.maxRec)) & (obj.fID ~= 0)
                str = sprintf( varargin{:} );
                if ~isempty( obj.title )
                    str = sprintf( '%s : %s', obj.title, str );
                end
                fprintf(obj.fID, str );
                if obj.counter == obj.maxRec
                    result = input( sprintf('>>> Debug record reached %d.\n    CR for another %d records, 0 to stop, or N to trace this amount > ', ...
                        obj.counter, obj.maxRec) );
                    if isempty(result) || (result ~= 0)
                        obj.counter = 0;
                        if result ~= 0
                            obj.maxRec = result;
                        end
                    elseif result == 0
                        obj.off;
                    end
                end
            end
        end
    end
end
