%
% September 2nd, 2015
%   explicit argument of k
%
% November 7th, 2013  Hiro Yamamoto  hiro@ligo.caltech.edu
%
%   first release based on SIS LaguerreGauss package
%

classdef LGTK < handle
    
    methods (Static)
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % 2D Laguerre Gaussian with invQ as input
        % LGField( p, m, x, y, invQ ) or
        % LGFieldRW( p, m, x, y, invRoC, w )
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        function E = LGField( p, m, x, y, invQ, k )
            [invRoC, w] = MMTK.qinv2RinvW( invQ, k );
            E = LGTK.LGFieldRW( p, m, x, y, invRoC, w, k );
        end
        
        function E = LGFieldRW( p, m, x, y, invRoC, w, k )
            [x2d, y2d] = meshgrid( x, y );
            rho = x2d.^2 + y2d.^2;
            A = -(1i*k/2*invRoC + 1/w^2)*rho;
            if m ~= 0
                A = A + 1i * m * atan2(y2d,x2d);
            end
            rho = sqrt(2) * sqrt(rho) / w;
            m = abs(m);
            E = sqrt( 2*factorial(p)/(pi*factorial(p+m)) ) / w ...
                * LGTK.LaguerrePoly(p,m,rho.^2).*exp(A);
            if m ~= 0
                E = E .* rho.^m;
            end
        end
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % Laguerre polynomial
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        function data = LaguerrePoly( p, m, r )
            switch p
                case 0
                    data = ones(size(r));
                    
                case 1
                    data = ( m + 1 ) - r;
                    
                case 2
                    data = ( r.*(r - 2*(m+2)) + (m+1)*(m+2) ) / 2;
                    
                 case 3
                     data = (r.*(r.*(3*(m+3)-r)-3*(m+2)*(m+3))+(m+1)*(m+2)*(m+3))/6;
                     
                otherwise
                    Apml = 1 / factorial(p);
                    if mod(p,2) == 1
                        Apml = -Apml;
                    end
                    data = Apml;
                    for l = 1 : p
                        data = data .* r;
                        Apml = -(p+1-l)*(p+m+1-l)/l * Apml;
                        data = data + Apml;
                    end
            end
        end
    end
end
