% 
% twiddle
%
% calculate analytic solution of cavity fields using scalar amplitudes.
% results are stored in FieldObj.Eamp
%
% September 2, 2015
%    changing ConstantTK.kLam0 to obj.ifo.fields.kCR
%
% October 26, 2014
%   handle of FFTIFO itself is saved in all manager objects, 
%   instead of handle of multiple manager handles
%

classdef twiddle < handle
    properties
        ifo
        name = 'unnamed'
        
        IFOmat  % Nfield x Nfield matrix, inverse is used for field calculation
        % Eamp    % field amplitude gussed by twiddle. Moved to FieldObj
    end
    
    methods
        % constructor
        function obj = twiddle( ifo )
            obj.ifo = ifo;
            obj.name = class(ifo);
            obj.IFOmat = [];
        end

        function buildMat( obj )
            totalFields = 0;
            % find total number of fields
            for iop = 1 : length( obj.ifo.optics.optList )
                numFields = length( obj.ifo.optics.optList(iop).inPorts ) ...
                    + length( obj.ifo.optics.optList(iop).outPorts );
                totalFields = totalFields + numFields;
            end
            % make identity matrix
            obj.IFOmat = zeros( totalFields, totalFields );
            % fill elements for reflection and transmission
            for iop = 1 : length( obj.ifo.optics.optList )
                if obj.ifo.optics.optList(iop).isMirrorObj
                    for portIndex = 1 : length( obj.ifo.optics.optList(iop).inPorts )
                        if obj.ifo.optics.optList(iop).isHRsideIndex( portIndex )
                            r = obj.ifo.optics.optList(iop).rHR;
                        else
                            r = obj.ifo.optics.optList(iop).rAR;
                        end
                        srcID = obj.ifo.optics.optList(iop).inPorts(portIndex);
                        sinkID = obj.ifo.optics.optList(iop).outPorts( obj.ifo.optics.optList(iop).reflMatch(portIndex) );
                        obj.IFOmat( sinkID, srcID ) = r;
                        sinkID = obj.ifo.optics.optList(iop).outPorts( obj.ifo.optics.optList(iop).transMatch(portIndex) );
                        obj.IFOmat( sinkID, srcID ) = obj.ifo.optics.optList(iop).t;
                    end
                end
            end
            % fill for propagation
            for ipr = 1 : length( obj.ifo.props.propList )
                spacePhase = ( obj.ifo.fields.kCR * obj.ifo.props.propList(ipr).delL + ...
                    obj.ifo.fields.kSB * obj.ifo.props.propList(ipr).L0 ) + obj.ifo.props.propList(ipr).lockPhase;
                obj.IFOmat( obj.ifo.props.propList(ipr).outPortID, obj.ifo.props.propList(ipr).inPortID ) = ...
                    exp( 1i*( obj.ifo.props.propList(ipr).lockPhase - spacePhase ) );
            end
        end
        
        function calcFields( obj, srcID )
            index = 1:size(obj.IFOmat);
            V = obj.IFOmat(:,srcID);
            M = obj.IFOmat;
            for il = size(M):-1:1
                if il == srcID || isempty( M(il,:) ~= 0 )
                    index(il:end-1) = index(il+1:end);
                    index(end) = [];
                    M(il,:) = [];
                    M(:,il) = [];
                    V(il,:) = [];
                end
            end
            Esrc = sqrt( obj.ifo.fields.fieldList(srcID).calcPower );
            E = (eye(length(V)) - M) \ V * Esrc;
%            E = inv(eye(length(V)) - M) * V * Esrc;
            for il = 1 : length(index)
                obj.ifo.fields.fieldList(index(il)).Eamp = E(il);
            end
        end

    end
end