%
% mich = MICHIFO;
%
% mich.fields.printAllInfo
%

classdef AdVirgoIMCIFO < FFTIFO
    
    methods
        function obj = AdVirgoIMCIFO( varargin )
            obj@FFTIFO( varargin{:} );
        end
        
        function defineIFO( obj, varargin )
            global DHS_offset DHS_dx DHS_dy
            
            RoCMC2 = 185.1;
            Ltot = 143.424 * 2;
            L13 = 86.25e-3;
            L12 = (Ltot - L13)/2;
            theta2 = atan( L13/2/L12 );
            theta1 = (pi/2 - theta2)/2;
            thick13 = 0.03;
            thick2 = 0.09;
            T_MC1 = 2314e-6;
            T_MC3 = 2410e-6;
            Lar_MC13 = 1900e-6;
            Lhr_MC1 = 4.5e-6;
            Lhr_MC3 = 9e-6;
            L_MC2 = 13e-6;
            T_MC2 = 2.2e-6;
            Aperture13 = 80e-3;  % 40mm aperture -> 40*sqrt(2) = 56mm
            Aperture2 = 140e-3;  % 10 x beam size
            mapWgt = 1;
            x0 = 0;
            y0 = 0;
            
            PcoatMC2 = 0; % coating absorption of MC2
            PcoatMC1 = 0;
            PcoatMC3 = 0;
            tiltY = 0;
            tiltX = 0;
            MC2type = 0;
            mirSet = 1;
            MC2map = './Data/MC_end_mirror_map_surface_2.col';
            useShield = 1;
            dHoles = 90e-3;
            
            offset = 0;
            dx = (dHoles - L13)/2;
            dy = 0;
            res2 = 0;
            
            % imc = AdVirgoIMCIFO('PcoatMC2', 0.2) or imc = AdVirgoIMCIFO('MC2map', '');
            eval( UtilTK.setVars( who, true, varargin{:} ) );
            
            DHS_offset = offset;
            DHS_dx = dx;
            DHS_dy = dy;
            
            if mirSet == 0
                MC1map = './Data/suba__S14018_2_50mmCA.col';
                MC3map = './Data/subb_50mm_S14018_1_CA.col';
            elseif mirSet == 1
                MC1map = './Data/c14065_10_subA_50mm.dat';
                MC3map = './Data/c14065_20_subB_50mm.dat';
            else
                MC1map = '';
                MC3map = '';
                if mirSet < 0
                    MC2map = '';
                end
            end
            
            Ltot = 2*L12 + L13;
            fprintf(1,'Ltot = %g, C/2L = %12.8f\n', Ltot, ConstantTK.c/Ltot);
            
            % IMC optics
            if MC2type == 0
                MC2Mirror = 'Mirror4x4Obj';
            else
                MC2Mirror = 'VirgoMC2';
            end
            
            obj.addLaser( 'src' );
            obj.addMirror( 'MC1', 'invRoC', 0, 'Aperture', Aperture13, ...
                'Thick', thick13, 'T', T_MC1, 'Lhr', Lhr_MC1, 'Lar', Lar_MC13, 'incAngle', theta1 );
            obj.addMirror( 'MC3', 'invRoC', 0, 'Aperture', Aperture13, ...
                'Thick', thick13, 'T', T_MC3, 'Lhr', Lhr_MC3, 'Lar', Lar_MC13, 'incAngle', theta1 );
            obj.addMirrorGeneric( MC2Mirror, 'MC2', 'invRoC', 1/RoCMC2, 'Aperture', Aperture2, 'resolution', res2, ...
                'Thick', thick2, 'T', T_MC2, 'Lhr', L_MC2, 'incAngle', theta2, 'optX0', x0, 'optY0', y0 ); % , 'virtualAperture', Aperture2+0.04 );
            obj.addDetector( 'refl' );
            obj.addDetector( 'trans' );
            
            % optics placements
            % MC1-HRx] => [MC3-HRy => MC3-HR-x] => [MC2-HRy => MC2-HRx] => [MC1-HRy
            obj.addProp( '', 'src', 'MC1-ARx' );
            obj.addProp( 'Prop13', 'MC1-HRx', 'MC3-HRy', L13 );
            obj.addProp( '', 'MC3-HRx', 'MC2-HRy', L12 );
            obj.addProp( '', 'MC2-HRx', 'MC1-HRy', L12 );
            obj.addProp( '', 'MC1-ARy', 'refl' );
            obj.addProp( '', 'MC3-ARy', 'trans' );
            
            if useShield > 0
                obj.setModMap( 'MC1', @obj.DIHEDRONShieldMC1 );
                obj.setModMap( 'MC3', @obj.DIHEDRONShieldMC3 );
            end
            
            % optics maps
            obj.setHRfiles('MC1', MC1map, ...
                'map*mapWgt + calcThermal(0, r, 0, PcoatMC1, wMC1, 0.02, 0.02 )', ...
                0, 1e20, 0, 'PcoatMC1', PcoatMC1, 'wMC1', 0.0051, 'mapWgt', mapWgt );
            obj.setHRfiles('MC2', MC2map, ...
                'map*mapWgt + r.^2/2*invRoC2 + calcThermal(0, r, 0, PcoatMC2, wMC2, 0.02, 0.07 ) + x*tiltX + y*tiltY', ...
                0.130/2, RoCMC2, 0, 'PcoatMC2', PcoatMC2, 'wMC2', 0.0108, 'tiltX', tiltX, 'tiltY', tiltY, 'invRoC2', 0, 'mapWgt', mapWgt );
            obj.setHRfiles('MC3', MC3map, ...
                'map*mapWgt + calcThermal(0, r, 0, PcoatMC3, wMC3, 0.02, 0.02 )', ...
                0, 1e20, 0, 'PcoatMC3', PcoatMC3, 'wMC3', 0.0051, 'mapWgt', mapWgt );
        end
        
        function setupBaseMode( obj, varargin )
            %{
            beam radius at input = 5.1107e-3 m
            wavefront is flat
            %}
            
            eval( UtilTK.setVars( who, true, varargin{:} ) );
            
            root = obj.optics.findPortID( 'src' );
            % small wave front curvature to place the waist inside of the MC
            % based on the simulation calculation without thermal effect
            qInv = -7.29384e-06 - 1i*0.0129713;
            obj.cavities.fillQvals( root, qInv );
        end
        
        function lock( obj, varargin )
            
            if ~isa( obj.optics.optList( obj.optics.findNamedOptic( 'MC2' )), 'VirgoMC2' )
                lock@FFTIFO( obj, varargin{:} );
                return;
            end
                
            if length( varargin ) >= 2
                errorLimit = varargin{2};
            else
                errorLimit = 1e-3;
            end
            
            if length( varargin ) >= 1
                repeat = varargin{1};
            else
                repeat = 2;
            end
            
            % line of coupling
            idCouple = find( abs([obj.cavities.guessList(1).masterPath(:).action]) == 7, 1 );
            idPump = find( abs([obj.cavities.guessList(1).masterPath(:).action]) == 4, 1, 'last' );
            
            % make the second loop convergence loose
            obj.cavities.guessList(2).errorLimit = errorLimit;
            obj.cavities.guessList(2).Nacc = 2;
            obj.cavities.guessList(2).Nave = 2;
            
            obj.cavities.guessList(1).masterPath(idCouple).action = -7;
            obj.cavities.guessList(1).masterPath(idPump).action = -4;
            
            for iloop = 1 : repeat
                % disable the second loop activation
                obj.cavities.guessList(1).masterPath(idCouple).action = -7;
                % lock the first loop
                lock@FFTIFO( obj, 0, 0, 0, iloop == 1 );
                
                P13 = obj.power('MC3-HRy-i');
                P31 = obj.power('MC3-HRy-o');
                Pout = obj.power('MC3-ARy-o');
                fprintf('%d : del = %.9g, P(MC1->MC3) = %.9g, P(MC3->MC1) = %.9g, Pout = %.9g\n', ...
                    iloop, obj.props.propList(3).delL, P13, P31, Pout );
                
                % activate coupling
                obj.cavities.guessList(1).masterPath(idCouple).action = 7;
                obj.calc( 0, 0, 0, 0 );
                
                P13 = obj.power('MC3-HRy-i');
                P31 = obj.power('MC3-HRy-o');
                Pout = obj.power('MC3-ARy-o');
                LG13 = obj.LGFrac('MC3-HRy-i',0:10,0);
                LG31 = obj.LGFrac('MC3-HRy-o',0:10,0);
                LG13(1) = 1 - LG13(1);
                LG31(1) = 1 - LG31(1);
                
                fprintf('%d : P(MC1->MC3) = %.9g, P(MC3->MC1) = %.9g, Pout = %.9g\n', ...
                    iloop, P13, P31, Pout );
                format long e
                LG13
                LG31
                
                % activate pumping in
                obj.cavities.guessList(1).masterPath(idPump).action = 4;
            end
        end
    end
    
    methods( Static )
        % implement the sheid in front of DIHEDRON
        % refl  {1}:HRx->HRy, {2}:HRy->HRx, {3}:ARx->ARy, {4}:ARy->ARx
        function [reflOut, transOut] = DIHEDRONShieldMC1( reflIn, transIn, x, y )
            [reflOut, transOut] = AdVirgoIMCIFO.shieldMask( 1, reflIn, transIn, x, y );
        end
        
        function [reflOut, transOut] = DIHEDRONShieldMC3( reflIn, transIn, x, y )
            [reflOut, transOut] = AdVirgoIMCIFO.shieldMask( 3, reflIn, transIn, x, y );
        end
        
        function [reflOut, transOut] = shieldMask( whichMC, reflIn, transIn, x, y )
            global DHS_offset DHS_dx DHS_dy
            
            shieldRadius = 0.02;
            transOut = transIn;
            reflOut = cell(size(reflIn));
            reflOut(3:4) = reflIn(3:4);
            
            if whichMC == 1
                clearAperture = ( (x-DHS_dx-DHS_offset).^2 + (y-DHS_dy).^2 ) < shieldRadius^2;
                reflOut{1} = fliplr(clearAperture) .* reflIn{1};
                reflOut{2} = reflIn{2} .* clearAperture;
            else
                clearAperture = ( (x-DHS_dx+DHS_offset).^2 + (y-DHS_dy).^2 ) < shieldRadius^2;
                reflOut{1} = reflIn{1} .* fliplr(clearAperture);
                reflOut{2} = clearAperture .* reflIn{2};
            end
        end
    end
end
