classdef FP0 < FFTIFO
    
    methods
        function obj = FP0( varargin )
            obj@FFTIFO( varargin{:} );
        end

        function defineIFO( obj, varargin )
            A = 0.34;
            Lcav = 4000;
            T = 1e-6;
            w = 0.09;
            tiltE = 0;
            
            y0 = 30;
            wRes = 0.0507;
            spM = false;
            lamCR = 1.064e-6;
            
            eval( UtilTK.setVars( who, true, varargin{:} ) );

            RoC = MMTK.wL2RoC( w, Lcav, -1, 2*pi/lamCR );
            RoCTM = MMTK.wL2RoC( wRes, Lcav, -1, 2*pi/lamCR );
            z = 2*pi/1.064e-6/(2*RoC*y0);
            y0 = y0 / (2*pi/1.064e-6);
            alpha = 0.175; beta = -0.05;
            
            %% add optics
            % laser source
            obj.addLaser( 'src', 'lamCR', lamCR );
            
            % ITM
            obj.addMirror( 'ITM', 'invRoC', 1/RoCTM, 'Aperture', A, 'Thick', 0, 'T', T, 'nRef', 1 );
            % ETM
            obj.addMirror( 'ETM', 'invRoC', 1/RoCTM, 'Aperture', A, 'Thick', 0, 'T', 0, 'resolution', 0.002 );
                        
            %% define connections among optics
            obj.addProp( '', 'src', 'ITM-AR' );
            obj.addProp( 'FPprop', 'ITM-HR', 'ETM-HR', Lcav);
            
            if ~spM
                obj.setHRfiles( 'ITM', '', 'y0*(1-exp(-z*r.^2-alpha*(z*r.^2).^2-beta*(z*r.^2).^3))', 0.17, 1e20, 0, 'y0', y0,'z', z, 'alpha', alpha, 'beta', beta );
                obj.setHRfiles( 'ETM', '', 'tiltE*y + y0*(1-exp(-z*r.^2-alpha*(z*r.^2).^2-beta*(z*r.^2).^3))', 0.17, 1e20, 0, 'y0', y0,'z', z, 'alpha', alpha, 'beta', beta, 'tiltE', tiltE );
            else
                obj.setHRfiles( 'ETM', '', 'tiltE*y', 0, 0, 0, 'tiltE', tiltE );
            end
        end
    end
end
