%% Study mode dependence of effective curvatures
%
%  ind  : input mode HG(ind, 0)
%  Pabs : absorption of ITM and ETM
%  RoC  : ITM and ETM mirror RoC
%  L    : cavity length
%
classdef FPIFOGouy < FFTIFO
    
    methods
        function obj = FPIFOGouy( varargin )
            obj@FFTIFO( varargin{:} );
        end
        %% this defines the IFO configuration
        function defineIFO( obj, varargin )

            Pabs = 0;
            ind = 0;
            
            L = 286.848;
            RoC = 185.1;
 
            eval( UtilTK.setVars( who, true, varargin{:} ) );

            %% add optics
            % laser source
            obj.addLaser( 'src', 'ind1', ind );
            
            
            % ITM
            obj.addMirror( 'ITM', 'invRoC', 1/RoC, 'Aperture', 0.14, 'Thick', 0.02, 'T', 0.01 );
            % ETM
            obj.addMirror( 'ETM', 'invRoC', 1/RoC, 'Aperture', 0.14, 'Thick', 0.02, 'T', 5e-6 );
            
            % detector behind ETM
            obj.addDetector( 'trans' );
            
            %% define connections among optics
            obj.addProp( '', 'src', 'ITM-AR' );
            obj.addProp( 'FPprop', 'ITM-HR', 'ETM-HR', L);
            obj.addProp( '', 'ETM-AR', 'trans' );
            
            wTM = MMTK.w1FP( 1/RoC, 1/RoC, L );
            
            obj.setHRfiles( 'ITM', '', 'calcThermal(0, r, 0, Pabs, wTM, 0.02, 0.07 )', 0, 0, 0, 'Pabs', Pabs, 'wTM', wTM );
            obj.setHRfiles( 'ETM', '', 'calcThermal(0, r, 0, Pabs, wTM, 0.02, 0.07 )', 0, 0, 0, 'Pabs', Pabs, 'wTM', wTM );
        end
    end
end
