classdef LHOPRMI < FFTIFO
    
    methods
        function defineIFO( obj )
            %% parameters
            dPRC12 = 16.6107; dPRC23 = 16.1647; dPRC3B = 19.5384;
            dCP = 0.10;  dBSCPX = 4.8046;  dBSCPY = 4.8497; dCPITM = 0.005;
            thetaPR2 = 0.79/180*pi;  thetaPR3 = 0.615/180*pi;  
            thetaSR2 = 0.87/180*pi;  thetaSR3 = 0.785/180*pi;
            
            rmloss = 1;
            ITMloss = 1;
            angleScale = 1;
            armScale = 1;

            armLoss = 1;
            Atm = 0.326;
            Apr3 = 0.262;
            Abs = 0.36985;
            dBS = 0.06041;
              
            %% add optics
            obj.addLaser( 'src' );
            obj.addMirror( 'PRM', 'invRoC', -1/10.948, 'Thick', 0.0749, ...
                'T', 2.99e-2, 'Lhr', (8+0.5)*1e-6, 'Lar', (35+4.508)*1e-6 );
            obj.addMirror( 'PRM2', 'invRoC', -1/4.543, ...
                'T', 229e-6, 'Lhr', 10.7e-6, 'inc', thetaPR2 );
            obj.addMirror( 'PRM3', 'invRoC', 1/36.021, 'Aperture', Apr3, ...
                'T', 5.3e-6, 'Lhr', 17e-6, 'incAngle', thetaPR3 );
                        
            BSID = obj.addMirror( 'BS', 'Aperture', Abs, 'Thickness', dBS, ...
                'T', 0.5, 'Lhr', 14.3e-6, 'Lar', 51.4e-6, 'incAngle', pi/4 );
            ITMXID = obj.addMirror( 'ITMX', 'invRoC', 1/1934, 'Aperture', Atm, 'Thick', 0.2 + dCP, ...
                'T', 1.39e-2, 'Lhr', 5.9e-6*armLoss, 'Lar', (264+1.1)*1e-6, 'removeTilt', true );
            ITMYID = obj.addMirror( 'ITMY', 'invRoC', 1/1934, 'Aperture', Atm, 'Thick', 0.2 + dCP, ...
                'T', 1.42e-2, 'Lhr', 12.4e-6*armLoss, 'Lar', (294+1)*1e-6, 'removeTilt', true );

            obj.addDetector( 'dark' );
 
            %% build BS maps
            BSNN = 6;
            
            switch BSNN
                case 2  % temporary at LLO, using normal data
                    obj.optics.optList(BSID).HRfiles.addMap( './Data/BS02_S1_CIT.dat', '', 0.16, 0, 1 );
                    obj.optics.optList(BSID).ARfiles.addMap( './Data/BS02_R2Fig.dat', '', 0.16, 0, -1 );
                    obj.optics.optList(BSID).TRfiles.addMap( './Data/BS02_SPTWE.dat', '', 0.16, 0, -1 );
                case 5  % final at LLO, using normal data
                    obj.optics.optList(BSID).HRfiles.addMap( './Data/BS05_S1_CIT.dat', '', 0.16, 0, 1 );
                    obj.optics.optList(BSID).ARfiles.addMap( './Data/xBS_05_R2_Figure.dat', '', 0.16, 0, -1 );
                    obj.optics.optList(BSID).TRfiles.addMap( './Data/xBS_05_SPTWE_Figure.dat', '', 0.16, 0, -1 );
                case 6  % LHO, using normal data
                    obj.optics.optList(BSID).HRfiles.addMap( './Data/BS06_S1_CIT.dat', '', 0.16, 0, 1 );
                    obj.optics.optList(BSID).ARfiles.addMap( './Data/xBS_06_R2_Figure.dat', '', 0.16, 0, -1 );
                    obj.optics.optList(BSID).TRfiles.addMap( './Data/xBS_06_SPTWE_Figure.dat', '', 0.16, 0, -1 );
                case 100 % BS02 45 degree dara
                    obj.optics.optList(BSID).RPYfiles.addMap( './Data/BS02_RPY.dat', '', 0.22, 0 );
                    obj.optics.optList(BSID).RSXfiles.addMap( './Data/BS02_RSX.dat', '', 0.22, 0 );
                    obj.optics.optList(BSID).TPXfiles.addMap( './Data/BS02_TPX.dat', 'map(:,1:end-5)=map(:,6:end);func=map;', 0.22, 0 );
                    obj.optics.optList(BSID).TSYfiles.addMap( './Data/BS02_TSY.dat', 'map(:,1:end-5)=map(:,6:end);func=map;', 0.22, 0 );
                case 101 % COMSOL thermal BS
                    obj.optics.optList(BSID).RPYfiles.addMap( '', 'calcBSatZ( ''w'', x(1,:), y(:,1), 0, 0 );', 0.22, 0 );
                    obj.optics.optList(BSID).RSXfiles.addMap( '', 'calcBSatZ( ''w'', x(1,:), y(:,1), 0.06, 0 );', 0.22, 0 );
                    obj.optics.optList(BSID).TPXfiles.addMap( '', 'calcBST( x(1,:), y(:,1), 0.06, 10, 1 );', 0.22, 0 );
                    obj.optics.optList(BSID).TSYfiles.addMap( '', 'calcBST( x(1,:), y(:,1), 0.06, 10, -1 );', 0.22, 0 );
            end
            
            % implement BS baffle
            obj.optics.optList(BSID).modMap = @obj.BSbaffle;
            
            %% ITMX
            obj.optics.optList(ITMXID).HRfiles.addMap( ...
                './Data/ITM03_CIT.dat', ...     % phase map data file
                'map + (x.^2+y.^2)/2*invRoC', ...   % formula to modify the map : change the RoC for simple TCS
                0.16, 1939.3, 0, ...                % BS RoC, 1937.9m, was measured in the central 16cm 
                'invRoC', 0);                       % default value of the parameter in the formula
            obj.optics.optList(ITMXID).TRfiles.addMap( './Data/xITM_03_SPTWE_Figure.dat', '', 0.16, -79.70e3);
            obj.optics.optList(ITMXID).TRfiles.addMap( './Data/xCP_01_SPTWE_Figure.dat', '', 0.16, 0);
            
            % ITMY, just 2300m HR surface
            obj.optics.optList(ITMYID).HRfiles.addMap( ...
                '', ...     % phase map data file
                '(x.^2+y.^2)/2*invRoC', ...   % formula to modify the map : change the RoC for simple TCS
                0.16, 0, 0, ...                % BS RoC, 1937.9m, was measured in the central 16cm 
                'invRoC', 1/2300);              % default value of the parameter in the formula
%             obj.optics.optList(ITMYID).TRfiles.addMap( './Data/xITM_11_SPTWE_Figure.dat', '', 0.16, 570.0e3);
%             obj.optics.optList(ITMYID).TRfiles.addMap( './Data/xCP_05_SPTWE_Figure.dat', '', 0.16, 0);

%% ETM section
% uncomment this block to attach arm
%             % ETMX
%             ETMXID = obj.addMirror( 'ETMX', 'invRoC', 1/2245, 'Aperture', Atm, 'Thick', 0.2, ...
%                 'T', 3.6e-6, 'Lhr', armLoss*8.3e-6, 'resolution', 2e-3 );
%             obj.optics.optList(ETMXID).HRfiles.addMap( './Data/ETM08_CIT.dat', 'map', 0.16, 2241.54);
%             
%             % ETMY
%             ETMYID = obj.addMirror( 'ETMY', 'invRoC', 1/2245, 'Aperture', Atm, 'Thick', 0.2, ...
%                 'T', 3.6e-6, 'Lhr', armLoss*9.2e-6, 'resolution', 2e-3 );
%             obj.optics.optList(ETMYID).HRfiles.addMap( './Data/ETM12_CIT.dat', 'map', 0.16, 2238.9);
%             
%             % arms
%             obj.props.addProp( '', 'ITMX',  1, 'ETMX', 1, 3994.5);
%             obj.props.addProp( '', 'ITMY',  1, 'ETMY', 1, 3994.5);
%

%% connect optics
            obj.props.addProp( '', 'src', 1, 'PRM', 2 );
            obj.props.addProp( '', 'PRM',  1, 'PRM2', 1, 16.6107 );
            obj.props.addProp( '', 'PRM2',  2, 'PRM3', 1, 16.1647 );
            obj.props.addProp( '', 'PRM3',  2, 'BS', 1, 19.5384 );
            
            obj.props.addProp( '', 'BS',  3, 'ITMX', 2, dBSCPX + dCPITM);
%            obj.props.addProp( '', 'BS',  2, 'ITMY', 2, dBSCPY + dCPITM);
            obj.addOneProp( '', 'BS',  2, 'ITMY' );
            obj.addOneProp( '', 'BS',  2, 'ITMY', 2 );
            obj.addOneProp( '', 'BS',  2, 'ITMY', 2, 0.2 );
            obj.addOneProp( '', 'BS',  2, 'ITMY', 2, 0.2, 1e-7 );


            % SRC props : uncomment this to add SRC
%             obj.props.addProp( '', 'SRM',  1, 'SRM2', 1, 15.7566 );
%             obj.props.addProp( '', 'SRM2',  2, 'SRM3', 1, 15.4435 );
%             obj.props.addProp( '', 'SRM3',  2, 'BS', 4, 19.368 );
%             obj.props.addProp( '', 'SRM',  2, 'dark', 1 );
% make sure the following one to be commented out when SRC is added
            obj.props.addProp( '', 'BS',  4, 'dark', 1 );
        end
                
%% this adjusts the PRM2-PRM3 length to make the PRC mode matches with the arm mode
           % simple mode injector
         function setupBaseMode( obj )
            ITMXARoutID = obj.optics.findPortID( 'ITMX', 2 );
            BSHRxoutID = obj.optics.findPortID( 'BS', 1 );
            PRM3xoutID = obj.optics.findPortID( 'PRM3', 1 );
            PRM2xoutID = obj.optics.findPortID( 'PRM2', 1 );

            qInvArmXAR = 1.45/1940 - i*1.064e-6/pi/0.053^2;
            
            % find the adjustment of the PRM3-PRM2 length
            LadjP = obj.cavities.adjustModeMatch( qInvArmXAR, PRM3xoutID, 0, ...
                [ITMXARoutID, BSHRxoutID, PRM3xoutID, PRM2xoutID ], true );
            
            sprintf('PRM3-PRM2 is adjusted by %g', LadjP )
            
            obj.cavities.fillQvals( ITMXARoutID, qInvArmXAR );
         end
    end    
    
    methods( Static )
%% implement a BS baffle
        function [reflOut, transOut] = BSbaffle( reflIn, transIn, x, y )
            
            baffleMask = x.^2/0.105^2+y.^2/0.13^2 < 1;
            
            reflOut{1} = baffleMask .* reflIn{1};
            reflOut{2} = reflIn{2} .* fliplr(baffleMask);
            reflOut{3} = reflIn{3} .* fliplr(baffleMask);
            reflOut{4} = baffleMask .* reflIn{4};
            transOut{1} = baffleMask .* transIn{1};
            transOut{2} = transIn{2} .* fliplr(baffleMask);
            transOut{3} = transIn{3} .* fliplr(baffleMask);
            transOut{4} = baffleMask .* transIn{4};
        end
    end
end