%% object defining the setup
classdef TMRef < FFTIFO
    
    methods
        %% to support runtime variables
        function obj = TMRef( varargin )
            obj@FFTIFO( varargin{:} );
        end

        %% definition of IFO
        function defineIFO( obj, varargin )
            
            %% variables, which can be changed interactively by UtilTK.setVars
            Ldet = 3994.5;     % distance between the mirror and detector
            RoC = 2245; % RoC of the mirror
            A = 0.33;
            
            %% this is to change variables defined above by using runtime argument, 
            % like mir = MirrorRef( 'incAng', 0.043 / 143.424 );
            eval( UtilTK.setVars( who, true, varargin{:} ) );
            
            %% setup optics
            % add laser. "help FFTIFO.addLaser" will show all parameters
            obj.addLaser( 'src' );
            
            obj.addMirror( 'TM', 'invRoC', 1/RoC, 'Thickness', 0.0, 'T', 0, 'Aperture', A );
            
            % add detector
            obj.addDetector( 'reflMon' );
            
            %% connect optics   : help FFTIFO.addProp
            % addProp(A,B) calls addOneProp(A,B,Lab) and addOneProp(B,A,Lab) if
            % propagation from B to A makes sense
            obj.addOneProp( '', 'src-out', 'TM-HR' );
            obj.addProp( '', 'TM-HR', 'reflMon-1', Ldet );  % detector name is somehow 'name-1', bad legacy to be fixed

        end
        
        %% define the input beam mode
        % fillQvals( fieldID, 1/qVal ) specified the mode. Then this mode is
        % propagated to all fields and this mode base is used for mode
        % calculations, like HGFrac.
        % This setting is necessary because there is no cavity which can be used to
        % define a default mode. For an optical system which has a
        % reasonable cavity which can be used to define modes, that is used
        % as the default mode.
        function setupBaseMode( obj, varargin )
            RoC = 2245;
            w = 0.062;
            eval( UtilTK.setVars( who, true, varargin{:} ) );
            
            % invq : 1/R - i lambda/pi 1/w^2 => -1/R - i lambda/pi 1/w^2
            qInv = 1/RoC - 1i*1.064e-6/pi/w^2;
            % field ID of laser source
            root = obj.optics.findLaser;
            % mode is defined by specifying the field ID and the q value
            obj.cavities.fillQvals( root, qInv );
        end
    end
end