% function data = interpCMSL( modelMPH, symbol, x, y, z, slope )
%
% calculate expression symbol, like T-T_amb or w, at (x+slope*z,y,z)
% default slope = 0, and z can be an array.
%
% z is usually from fron to back, so use -interpCMSL( 'w' ) for the HR surface height
%
% The index ordering is that of matlab, i.e., the first index is for y, and second index for x.
%
% The data( yj, xi, zk ) is the value at (x,y,z) = (xi+slope*zk, yj, zk), 
% i.e., along a line with finite slope, like the temperature along a beam line in a BS.
%
%   example : 
%       interpCMSL( 'ETM.mph', ''w'', x(1,:), y(:,1), 0 )
%       interpCMSL( 'BS.mph', 'T-T_amb', x, y, zlist, tanN ) * dndt;
%
function data = interpCMSL( modelMPH, symbol, x, y, z, slope )

if ~exist( 'slope', 'var' )
    slope = 0;
end
if ~exist( 'z', 'var' )
    z = 0;
end

model = mphload(modelMPH);
for iz = 1 : length(z)
    [xx,yy,zz] = meshgrid( x+slope*z(iz), y, z(iz) );
    xxx = [xx(:), yy(:), zz(:)]';
    dataZ = mphinterp( model, symbol, 'coord', xxx );
    data(:,:,iz) = reshape( dataZ, length(y), length(x) );
end
