function str = oneAnalPRM( ifo )
% [str, DfromY, DfromX, xBS] = oneAnalPRM( ifo )

    % field ID of dark and bright port on BS
    IDDark = ifo.fields.findNamedField('BS-ARy-o');
    IDBright = ifo.fields.findNamedField('BS-HRx-o');

    % power reflected by PRM
    reflPower = ifo.power('PRM-AR-o');

    % CD = power on dark port / power on bright
    CD = ifo.power('BS-ARy-o') / ifo.power('BS-HRx-o');
    
    % fit the field by a gaussian form
    % cwR has the fitting result of the field by gaussian, field = gauss1D(x)*gauss1D(y)
    %     gauss1D(x) = exp( -(x-c0)^2 ( 1/w^2 + i invRk / 2 )
    % info is string showing the result
    [info,cwR6] = ifo.gaussFit('BS-HRy-i');
    sprintf('field from X arm gauss fit : %s', info )
    [info,cwR4] = ifo.gaussFit('BS-ARx-i');
    sprintf('field from Y arm gauss fit : %s', info )

    % loss by BS geometry
    loss2x = 1 - 2*ifo.power('BS-ARx-o') / ifo.power('BS-HRx-i');
    loss2y = 1 - 2*ifo.power('BS-HRy-o') / ifo.power('BS-HRx-i');

    % now, we are calculating something which change fields on the BS, so save fields to restore
    Ebright = ifo.fields.fieldList(IDBright).EFFT;
    Edark = ifo.fields.fieldList(IDDark).EFFT;

    % calculate the transmission of the field coming from Y arm to dark port
    ifo.trans('BS-HRy-i');
    % now the field in BS-ARy-o contains only the transmission from Y arm, 
    % do no include the reflection from the X arm
    lossDfromY = 1 - 2*ifo.power('BS-ARy-o-trans') / ifo.power('BS-HRy-i');    
    % calculate the reflection from Y arm to bright
    ifo.refl('BS-HRy-i');
    % now the field in the bright port contains only the reflection from Y arm
    lossBfromY = 1 - 2*ifo.power('BS-HRx-o-refl') / ifo.power('BS-HRy-i');
    
    % relfection from tha X arm to dark port
    ifo.refl('BS-ARx-i');
    lossDfromX = 1 - 2*ifo.power('BS-ARy-o-refl') / ifo.power('BS-ARx-i');
    % tranmission from X arm to bright port
    ifo.trans('BS-ARx-i');
    lossBfromX = 1 - 2*ifo.power('BS-HRx-o-trans') / ifo.power('BS-ARx-i');

    % restore the dark and bright port fields
    ifo.fields.fieldList(IDBright).EFFT = Ebright;
    ifo.fields.fieldList(IDDark).EFFT = Edark;

    % higher order mode fraction on the bright port of BS
    HOM = 1-abs(ifo.HGCoef('BS-HRx-o',0,0))^2/ifo.power('BS-HRx-o');

    % print bunch of information
    str = sprintf('CD = %g (ppm),  Power = %g, Arm P ave = %g, HOM(bright) = %g (ppm),  Refl = %g\n loss(Y->dark) = %g, loss(Y->bright) = %g, loss(X->datk) = %g, loss(X->bright) = %g\n %g %g %g %g %g %g\n', ...
        CD*1e6, ifo.power('PRM-HR-o'), (ifo.power('ITMX-HR-o')+ifo.power('ITMY-HR-o'))/2, HOM*1e6, reflPower, ...
        lossDfromY, lossBfromY, lossDfromX, lossBfromX, ...
        loss2x, loss2y, cwR6(1).w, cwR6(2).w, ...
        1/ (cwR6(1).invRk / (2*pi/1.064e-6)), 1/ (cwR4(1).invRk / (2*pi/1.064e-6)) ...
        );

    % if ETM exists, print information about the arm
    if ifo.fields.findNamedField('ETMX-HR-o', true) > 0

        % LG mode on the bright port
        sprintf('LGCoef(0:5,0) of Bright Port')
        abs(ifo.LGCoef('BS-HRx-o', 0:5,0)).^2/ifo.power('BS-HRx-o')

        % information about X arm
        [~,cwR6] = ifo.gaussFit('ITMX-HR-i');
        armID = ifo.fields.findNamedField('ITMX-HR-i');
        sprintf('X arm : power = %g, roundtrip loss (ppm) = %g, HOM(ppm) = %g\nbeam on ITM = (%g, %g) (cm)\n', ...
            ifo.power(armID), ifo.cavities.guessList(3).roundtripLoss*1e6, (1-abs(ifo.LGCoef(armID,0,0)).^2/ifo.power(armID))*1e6, ...
            cwR6(1).c0*100, cwR6(2).c0*100 )
        abs(ifo.LGCoef(armID, 0:5,0)).^2/ifo.power(armID)

        % information about Y arm
        [~,cwR6] = ifo.gaussFit('ITMY-HR-i');
        armID = ifo.fields.findNamedField('ITMY-HR-i');
        sprintf('Y arm : power = %g, roundtrip loss (ppm) = %g, HOM(ppm) = %g\nbeam on ITM = (%g, %g) (cm)\n', ...
            ifo.power(armID), ifo.cavities.guessList(2).roundtripLoss*1e6, (1-abs(ifo.LGCoef(armID,0,0)).^2/ifo.power(armID))*1e6, ...
            cwR6(1).c0*100, cwR6(2).c0*100 )
        abs(ifo.LGCoef(armID, 0:5,0)).^2/ifo.power(armID)
    end

end
