function runFPKAGRA( inPower )

% many helps defines in FFTIFO object
help FFTIFO
help FFTIFO.addMirror
help FFTIFO.setHRfiles

% build FP object
%% This is cold cavity case with 2.5kW input
%% make the cold one again
fp = FPKAGRA('power', inPower);

fp.lock
oneInfo( fp, '*** cold FP', 'ETMX-HR-o');

% this is the cold state surface map
ETMHR0 = fp.optics.getHRmap('ETMX', true);
ITMHR0 = fp.optics.getHRmap('ITMX', true);

% cold state ITM transmission map, i.e., the lens by cold state HR RoC
ITMTRX0 = fp.optics.optList(2).TPXmap;

% loop until stationary state is found
fp = thermalLoop( fp );

% show all necessary information
fp.printAll
% fp.fields.printAll
% fp.optics.printAll

% show field info, field name = 'optic name' - 'HR' or 'AR' - 'i' (for going in) or 'o' (for out going)
oneInfo( fp, '*** hot FP', 'ETMX-HR-o');

% field complex amplitude going to ITM HR. E(iy,ix) is the amplitude at xl(ix,iy)
figure(1)
[E, xl] = fp.getField('ITMX-HR-i');
pcolor( xl, xl, log(abs(E).^2) ); shading flat; axis equal; colorbar;

% ITM and ETM HR surface map
% ETMHR(iy,ix) is the height at x = xl(ix), y = xl(yl). This returns maps without nominal RoC
[ETMHR1, ETMxl] = fp.optics.getHRmap('ETMX', true);
[ITMHR1, ITMxl] = fp.optics.getHRmap('ITMX', true);

% ITM transmission map by laser heating
ITMTRX1 = fp.optics.optList(2).TPXmap;

% calculate the power term to find the optimal RH correction
% this calculates zernike terms, also removes ...
help zernikeCoef

% use 1.5xw radius central region to find the best curvature
% 3 means to calculate piston, tiltx, tilty and power
[~,cwR] = fp.gaussFit('ETMX-HR-i'); w=(cwR(1).w+cwR(2).w)/2;
[~,coef] = zernikeCoef( ETMHR1, ETMxl, ETMxl, w*1.5, 3);
% this is the effective RoC of ETM surface
ETMRoC = (w*1.5)^2/(4*coef(4));
% do the same for ITM
[~,cwR] = fp.gaussFit('ITMX-HR-i'); w=(cwR(1).w+cwR(2).w)/2;
[~,coef] = zernikeCoef( ITMHR1, ITMxl, ITMxl, w*1.5, 3);
ITMRoC = (w*1.5)^2/(4*coef(4));
% print RoCs
fprintf(1, 'zernike fit of the thermally distorted surface : dRoC(ITM) = %g, dRoC(ETM) = %g\n\n', ITMRoC, ETMRoC);

%% now change the parameters of the ITM and ETM to add the RH correction
% see the formulas in setHRfiles for ITM and ETM
% thermal heating changes the RoC to ITMRoC, so RH subtracts it
% This parameter is the aLIGO case ratio of the HR RoC correction to the power of the lens in ITM
% (1/HR RoC_by_RH - 1/HR RoC_no_RH) * RHRoC2Lens = 1/RoC of the lens induced by RH in ITM
% HR RoC becomes smaller, and the lens in ITM is diverging
% invRoCITMXRH should be positive to make the RoC smakker
% invRoCITMXCP should be negative for add a diverging lens. 
% So the sign of these two should be opposite
RHRoC2Lens = 11.1;
fp.updateMirMaps('ITMX', 'invRoCITMXRH', -1/ITMRoC, 'invRoCITMXCP', 1/ITMRoC*RHRoC2Lens);
fp.updateMirMaps('ETMX', 'invRoCETMXRH', -1/ETMRoC);

% lock and see the mode content in the arm
% loop until stationary state is found
fp = thermalLoop( fp );

oneInfo( fp, '*** hot FP with RH', 'ETMX-HR-o');

% save the surface map
ETMHRRH = fp.optics.getHRmap('ETMX', true);
ITMHRRH = fp.optics.getHRmap('ITMX', true);

% ITM transmission map with RH
ITMTRRH = fp.optics.optList(2).TPXmap;

%% summary plot
figure(2)
dITMHR = ITMHR1 - ITMHR0;
dITMHR = dITMHR - dITMHR(end/2,end/2);
dETMHR = ETMHR1 - ETMHR0;
dETMHR = dETMHR - dETMHR(end/2,end/2);

plot( ITMxl, ITMHR0(end/2,:)*1e9, 'b', ITMxl, ITMHR1(end/2,:)*1e9, 'bx', ITMxl, ITMHRRH(end/2,:)*1e9, 'b.', ITMxl, dITMHR(end/2,:)*1e9', 'b^', ...
    ETMxl, ETMHR0(end/2,:)*1e9, 'r', ETMxl, ETMHR1(end/2,:)*1e9, 'rx', ETMxl, ETMHRRH(end/2,:)*1e9, 'r.', ETMxl, dETMHR(end/2,:)*1e9', 'r^')
axis([-0.16 0.16 -40 10])
legend('ITM cold', 'ITM hot', 'ITM hot+RH', 'ITM hot-cold', 'ETM cold', 'ETM hot', 'ETM hot+RH', 'ETM hot-cold', 'location', 'south');

figure(3)
dITMTRX1 = ITMTRX1 - ITMTRX0;
dITMTRRH = ITMTRRH - ITMTRX0;

% transmission map = negative of optical path length, so plot them with sign changed
plot( ITMxl, -ITMTRX0(end/2,:)*1e9, ITMxl, -dITMTRX1(end/2,:)*1e9, ITMxl, -dITMTRRH(end/2,:)*1e9 )
axis( [-0.17 0.17 -100 100])
legend('cond state lens', 'lens by laser heating', 'lens by laser and ring heater')
title('ITM optical path length distrinution');

function oneInfo( fp, message, fieldName )
fprintf(1,'\n\n%s : Inforamtion about field %s\n', message, fieldName);
% power
P = fp.power( fieldName );
% LG mode fraction
LGfrac = abs(fp.LGCoef(fieldName, 0:5,0)).^2 / P;
% round trip loss
RTL = fp.roundtripLoss( 'ETMX-HR-o' );
% curvature of the reflected CR field (prompt reflection + leak from the cavity)
[~,cwR] = fp.gaussFit('ITMX-AR-o'); RoCCR = 2*pi/1.064e-6*(1/cwR(1).invRk + 1/cwR(2).invRk)/2;
% curvature of the reflected SB field, i..e, prompt reflection only
SBRefID = fp.refl( 'ITMX-AR-i' );
[~,cwR] = fp.gaussFit(SBRefID); RoCSB = 2*pi/1.064e-6*(1/cwR(1).invRk + 1/cwR(2).invRk)/2;
SB00 = fp.LGFrac(SBRefID,0,0);

fprintf(1,'power = %g, HOM = %g, RTL = %g, RoC(CR) = %g, RoC(SB) = %g, HOM(SB) = %g\nLG(1:5,0) fraction %g, %g, %g, %g, %g\n\n', P, 1-LGfrac(1), RTL, RoCCR, RoCSB, 1-SB00, LGfrac(2:6) );

