% main program to call calcThermal

function dat = thermalTest( coating, substrate )

%Dr. Muzammil A. Arain
%
%LIGO Group
%Department of Physics
%University of Florida
%PO Box 118440
%Gainesville FL 32611-8440
%
%Room No: NPB-2365
%Tel: (352) 392-4947
%
%
% Program that calculates thermal aberration in the TEST MASS  based on
% Hello-Vinet theory 

% Input Section for Power level and Beam size
Parm = 770e3;
Prc = 2.5e3;
Pc=0.3E-6*Parm*coating;    % Power Absorbed in the Coatings 0.5 ppm == 0.425 W
w=5.31E-2;         % Nominal Beam Radius
thickness=20E-2;    % Thicknes of substrate
radius=17E-2;       % Radius of fused silica mirrors
n=1.5;           % Refractive Index
alphas=2.5e-6;    % absorption of aLIGO fused silica substrate  2.5 ppm / m
resol=radius/100;   % Resolution of points along radial direction
%

Ps=Prc*thickness*alphas*substrate;   % Power Absorbed in the substrate 84 mW

rlist = 0:resol:radius;
dat = HVThermal( 2, rlist, Ps, Pc, w, thickness, radius );

totalaberrations=dat(:,2); % +dat(:,1)*(n-1);  

d0Thr = 6.696e-8;
w0Thr = 101.233;
b2Thr = -11.08;
b4Thr = 4845.66;
rr = rlist.^2;
pw = d0Thr .* exp( -w0Thr * rr ) .* ( 1 + b2Thr * rr + b4Thr * rr .* rr ) - d0Thr;

figure(1)
plot( rlist,dat(:,1)*5, 'b', rlist, pw*5, 'g', rlist, dat(:,2), 'r', rlist, totalaberrations, 'k');
xlabel('Radial Distance (m)');
ylabel('Aberrations (m)');
grid on;
legend('Surface Deformation','Phi', 'Substrate Aberrations','Total Aberrations in Transmission','Location','South');

dat2 = HVThermal( 0, rlist, Ps, Pc, w, thickness, radius );
figure(2)
plot( rlist, dat2, rlist, pw );

% Ref 1 
% Model of Thermal Wavefront Distortion in Interferometric Gravitational-Wave
% Detectors I: Thermal Focusing
% R. G. Beausoleil,1, ? E. D?Ambrosio,2 W. Kells,2 J. Camp,2 E. K. Gustafson,1 and M. M. Fejer1
% 1Edward L. Ginzton Laboratory, Stanford University, Stanford, CA
% 94305
return
